//-----------------------------------------------
//
// Main file for the Fl_MoxGen program, a program which will
// generate the dimensions of a Moxon Rectangle antenna when
// given the resonant frequency and the wire gauge.
//
// Copyright 2009 Andy Stewart
//
// Algorithm and diagram: L.B. Cebik, W4RNL (SK)
//
// Code for the Standard Wire Gauge was contributed by
// Colin Tuckley (G8TMV), 20-jul-2010.  Thanks, Colin!
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 3 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, see
// <http://www.gnu.org/licenses/>.
//
// Please report all bugs and problems to the author:
//
// Andy Stewart (KB1OIQ)
// kb1oiq@arrl.net
// http://home.comcast.net/~andystewart
//
//-----------------------------------------------

#include "fl_moxgen_defines.h"
#include "fl_moxgen_version.h"
#include "write_pdf.h"

#include <cstdlib>    // for exit(0)
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h> // for stat()
#include <time.h>
#include <unistd.h>   // for getcwd

#include <FL/Fl.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Help_Dialog.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_PNG_Image.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Shared_Image.H>
#include <FL/Fl_Window.H>

using namespace std;

#define WIDTH 675
#define HEIGHT 450

// Conversion constants for SWG -> inches since it's not a simple function
// SWG 0 -> 20 (notice the lack of 4 and 5, denoted by 9.999 as a placeholder)
static double swg[] = {0.324, 0.300, 0.276, 0.252, 9.999,
		       9.999, 0.192, 0.176, 0.160, 0.144,
		       0.128, 0.116, 0.104, 0.092, 0.080,
		       0.072, 0.064, 0.056, 0.048, 0.040,
		       0.036};

//-----------------------------------------------
// Simple Window class
//-----------------------------------------------

class SimpleWindow : public Fl_Window {

public:
  SimpleWindow(int w, int h, const char *title);
  ~SimpleWindow();

  // Input boxes
  Fl_Float_Input *Frequency;
  Fl_Float_Input *WireSize;
  Fl_Choice      *WireSizeUnits;

  // Buttons
  Fl_Button      *Calculate;
  Fl_Button      *QuitButton;

  // Inputs: Radio buttons for choosing the output units
  Fl_Group        *Button_Group;
  Fl_Round_Button *Feet_button;
  Fl_Round_Button *Inches_button;
  Fl_Round_Button *Meters_button;
  Fl_Round_Button *Millimeters_button;

  // Outputs: Shows each dimension
  Fl_Output *Dim_A;
  Fl_Output *Dim_B;
  Fl_Output *Dim_C;
  Fl_Output *Dim_D;
  Fl_Output *Dim_E;

  // Outputs: Boxes for displaying the output units
  Fl_Box *Box_A;
  Fl_Box *Box_B;
  Fl_Box *Box_C;
  Fl_Box *Box_D;
  Fl_Box *Box_E;

  // Output:  Box for displaying the pretty antenna picture
  Fl_Box *MoxpicBox;
  Fl_PNG_Image *MoxPic;

  char moxpic_file[100];
  char cwd[100];
  struct stat buf;

  // MenuBar
  Fl_Menu_Bar *MenuBar;

  // Misc
  double wd_in;

private:
  static void cb_Calculate(Fl_Widget *w, void *v);
  inline void cb_Calculate_i(Fl_Widget *w, void *v, int i);

  static void cb_Quit(Fl_Widget *w, void *v);
  inline void cb_Quit_i(void *v);

  static void cb_HelpAbout(Fl_Widget *w, void *v);
  inline void cb_HelpAbout_i(void *v);

  static void cb_HelpGettingStarted(Fl_Widget *w, void *v);
  inline void cb_HelpGettingStarted_i(void *v);

  // Each of these calls cb_Calulate_i with a unique integer value
  static void cb_Frequency(Fl_Widget *w, void *v);
  static void cb_WireSize(Fl_Widget *w, void *v);
  static void cb_WireSizeUnits(Fl_Widget *w, void *v);
  static void cb_FeetButton(Fl_Widget *w, void *v);
  static void cb_InchesButton(Fl_Widget *w, void *v);
  static void cb_MetersButton(Fl_Widget *w, void *v);
  static void cb_MillimetersButton(Fl_Widget *w, void *v);

  static void cb_WriteModelHoriz(Fl_Widget *w, void *v);
  inline void cb_WriteModelHoriz_i(void *v);
  static void cb_WriteModelVert(Fl_Widget *w, void *v);
  inline void cb_WriteModelVert_i(void *v);
  inline void gen_xnec2c_model(const char *filename, int horiz);

  static void cb_WritePDF(Fl_Widget *w, void *v);
  inline void cb_WritePDF_i(void *v);

  // General purpose functions
  inline double get_diam_wl(double freq, double wd, int units, double *wd_in);
  inline const char *get_wire_size_units();
  inline const char *get_output_dimensions();

};

//-----------------------------------------------
// main routine
//-----------------------------------------------

int main() {

  fl_register_images();

  SimpleWindow win(WIDTH, HEIGHT, "Fl_MoxGen");
  return (Fl::run());

}

//-----------------------------------------------
// Simple Window Constructor
//-----------------------------------------------

SimpleWindow::SimpleWindow(int w, int h, const char *title):Fl_Window(w,h,title) {

  FILE *outfile;

  begin();

    Frequency = new Fl_Float_Input  (140, 50, 100, 30, "Frequency (MHz)");
    Frequency->callback(cb_Frequency, this);
    Frequency->tooltip("Frequency in MHz");

    WireSize = new Fl_Float_Input  (320, 50, 100, 30, "Wire Size");
    WireSize->callback(cb_WireSize, this);
    WireSize->tooltip("Diameter of the wire");

    WireSizeUnits   = new Fl_Choice (430, 50, 70, 30, "");
    WireSizeUnits->add("AWG"); // 0
    WireSizeUnits->add("in");  // 1
    WireSizeUnits->add("mm");  // 2
    WireSizeUnits->add("SWG"); // 3
    WireSizeUnits->value(0);
    WireSizeUnits->callback(cb_WireSizeUnits, this);

    Calculate  = new Fl_Button (525, 50,  100, 30, "&Calculate");
    Calculate->callback(cb_Calculate, this);
    Calculate->tooltip("Compute the answer!");

    Dim_A      = new Fl_Output (525,  90, 100, 30, "A");    
    Dim_B      = new Fl_Output (525, 130, 100, 30, "B");
    Dim_C      = new Fl_Output (525, 170, 100, 30, "C");
    Dim_D      = new Fl_Output (525, 210, 100, 30, "D");
    Dim_E      = new Fl_Output (525, 250, 100, 30, "E");

    Box_A      = new Fl_Box    (630,  90,  1, 30, "in");
    Box_B      = new Fl_Box    (630, 130,  1, 30, "in");
    Box_C      = new Fl_Box    (630, 170,  1, 30, "in");
    Box_D      = new Fl_Box    (630, 210,  1, 30, "in");
    Box_E      = new Fl_Box    (630, 250,  1, 30, "in");

    Box_A->align(FL_ALIGN_RIGHT);
    Box_B->align(FL_ALIGN_RIGHT);
    Box_C->align(FL_ALIGN_RIGHT);
    Box_D->align(FL_ALIGN_RIGHT);
    Box_E->align(FL_ALIGN_RIGHT);

    QuitButton = new Fl_Button (525, 400, 100, 30, "&Quit");
    QuitButton->callback(cb_Quit, (void *)"Quit Button Pressed");

    Button_Group = new Fl_Group(525, 300, 100, 90, "Result Units");
    Button_Group->box(FL_DOWN_BOX);

      Inches_button = new Fl_Round_Button(525, 310, 64, 15, "Inches");
      Inches_button->type(102);
      Inches_button->down_box(FL_ROUND_DOWN_BOX);
      Inches_button->setonly();
      Inches_button->callback(cb_InchesButton, this);

      Feet_button = new Fl_Round_Button(525, 330, 64, 15, "Feet");
      Feet_button->type(102);
      Feet_button->down_box(FL_ROUND_DOWN_BOX);
      Feet_button->callback(cb_FeetButton, this);

      Millimeters_button = new Fl_Round_Button(525, 350, 64, 15, "Millimeters");
      Millimeters_button->type(102);
      Millimeters_button->down_box(FL_ROUND_DOWN_BOX);
      Millimeters_button->callback(cb_MillimetersButton, this);

      Meters_button = new Fl_Round_Button(525, 370, 64, 15, "Meters");
      Meters_button->type(102);
      Meters_button->down_box(FL_ROUND_DOWN_BOX);
      Meters_button->callback(cb_MetersButton, this);

    Button_Group->end();

    MoxpicBox = new Fl_Box(13, 98, 490, 340, "");

    // The picture might live in the current working directory,
    // if the user hasn't yet installed the program.  Look
    // there first, and if not found, look where it should
    // have been installed.  Die horribly if the image
    // cannot be found.

    getcwd(cwd, sizeof(cwd));
    snprintf(moxpic_file, sizeof(moxpic_file), "%s/%s", cwd, "moxgen.png");
    //    fprintf(stderr, "File 1: %s\n", moxpic_file);

    if (!stat(moxpic_file, &buf)) {
      MoxPic = new Fl_PNG_Image(moxpic_file);
    }
    else {
      snprintf(moxpic_file, sizeof(moxpic_file), "%s/%s", SHAREDIR, "moxgen.png");
      //      fprintf(stderr, "File 2: %s\n", moxpic_file);
      if (!stat(moxpic_file, &buf)) {
	MoxPic = new Fl_PNG_Image(moxpic_file);
      }
      else {
	fl_alert("Cannot find file moxgen.png\nExpected it to be in:\n    %s or\n    %s\n%s", 
		 cwd, SHAREDIR,
		 strerror(errno));
	exit(1);
      }
    }
    
    MoxpicBox->image(MoxPic);

    MenuBar = new Fl_Menu_Bar(0, 0, WIDTH, 30);
    static Fl_Menu_Item MenuTable[] = {
      {"&File", 0, 0, 0, FL_SUBMENU},
        {"&Print to File", 0, cb_WritePDF, this},
        {"&Write Model", 0, 0, 0, FL_SUBMENU},
          {"&Horizontal Polarization", 0, cb_WriteModelHoriz, this},
          {"&Vertical Polarization", 0, cb_WriteModelVert, this},
          {0},

        {"&Quit", 0, cb_Quit},
        {0},

      {"&Help", 0, 0, 0, FL_SUBMENU},
        {"&About", 0, cb_HelpAbout},
        {"&Getting Started", 0, cb_HelpGettingStarted},
        {0},
      {0}
    };
    MenuBar->menu(MenuTable);

  end();

  // BOZO: Currently, the MoxPic image does not resize
  // thus making everything look funny.  All other
  // GUI elements seem to resize reasonably.
  // When this is fixed, uncomment the line below.

  //  resizable(this);
  show();
}

//-----------------------------------------------
// Simple Window Destructor
//-----------------------------------------------

SimpleWindow::~SimpleWindow() {

}

//-----------------------------------------------
// get_diam_wl()
//-----------------------------------------------

double SimpleWindow::get_diam_wl(double freq, double wd, int units, double *wd_in) {
  double zi;            // variable for AWG & SWG to inches conversion
  double wli;           // variable
  double dw;

  switch (units) {

  case 0: // AWG
    // first convert AWG to inches, then convert to diameter in wirelengths (dw)
    zi = 0.005 * pow(92, ( (36 - wd)/39 ));
    *wd_in = zi;         // save wire diameter in inches
    wli = 11802.71 / freq;
    dw = zi / wli;
    break;

  case 1: // in
    *wd_in = wd;        // save wire diameter in inches
    wli = 11802.71 / freq;
    dw = wd / wli;
    break;

  case 2: // mm
    *wd_in = wd / 25.4; // save wire diameter in inches
    wli = 299792.5 / freq;
    dw = wd / wli;
    break;

  case 3: // SWG
    // first convert SWG to inches, then convert to diameter in wirelengths (dw)
    // Suported: SWG 0 -> 3, and 6 -> 20 (4 and 5 don't exist)
    if (((wd >= 0) && (wd <= 20)) && (wd != 4) && (wd != 5)) {
      zi = swg[(int)wd];
    }
    else {
      fl_message("Unexpected SWG value.\nPlease enter a value of 0, 1, 2, 3, 6, ..., 20");
      zi = 0;
    }
    *wd_in = zi;         // save wire diameter in inches
    wli = 11802.71 / freq;
    dw = zi / wli;
    break;

  default: // can't get here
    fprintf(stderr, "Can't get to default case!!!");
    break;

  } // switch (units)

  return dw;

}

//-----------------------------------------------
// Callback: Quit
//-----------------------------------------------

void SimpleWindow::cb_Quit (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Quit_i(v);
}

void SimpleWindow::cb_Quit_i(void *v) {
  exit(0);
}

//-----------------------------------------------
// Callbacks
//-----------------------------------------------

void SimpleWindow::cb_Frequency (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 1);
}

void SimpleWindow::cb_WireSize (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 2);
}

void SimpleWindow::cb_WireSizeUnits (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 3);
}

void SimpleWindow::cb_Calculate (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 4);
}

void SimpleWindow::cb_FeetButton (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 5);
}

void SimpleWindow::cb_InchesButton (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 6);
}

void SimpleWindow::cb_MetersButton (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 7);
}

void SimpleWindow::cb_MillimetersButton (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_Calculate_i(w, v, 8);
}

void SimpleWindow::cb_Calculate_i(Fl_Widget *w, void *v, int i) {
  // int i is used to determine who called this function
  // i = 1:  Frequency input
  // i = 2:  WireSize input
  // i = 3:  WireSizeUnits input
  // i = 4:  Calculate button
  // i = 5:  Feet radio button
  // i = 6:  Inches radio button
  // i = 7:  Meters radio button
  // i = 8:  Millimeters radio button

  static int units;
  const char *units_str[] = { "AWG", "in", "mm" };

  static char a_str[20];
  static char b_str[20];
  static char c_str[20];
  static char d_str[20];
  static char e_str[20];

  static double freq  = 0;     // frequency
  static double wd    = 0;     // wire diameter

  static double dw;            // diameter in wavelengths
  static double d1;            // variable

  static double wf;            // convert wavelengths to feet
  static double wi;            // convert wavelengths to inches
  static double wmm;           // convert wavelengths to millimeters

  static double a, b, c, d, e; // antenna dimensions

  const static double aa = -0.0008571428571;
  const static double ab = -0.009571428571;
  const static double ac =  0.3398571429;

  const static double ba = -0.002142857143;
  const static double bb = -0.02035714286;
  const static double bc =  0.008285714286;

  const static double ca =  0.001809523381;
  const static double cb =  0.01780952381;
  const static double cc =  0.05164285714;

  const static double da =  0.001;
  const static double db =  0.07178571429;

  switch (i) {

  case 1: // Frequency
    freq = atof( Frequency->value() );
    wf  = 983.5592 / freq;
    wi  = 11802.71 / freq;
    wmm = 299792.5 / freq;
    break;
    
  case 2: // WireSize (diameter)
    wd = atof( WireSize->value() );
    break;

  case 3: // WireSizeUnits
    units = ( WireSizeUnits->value() );
    break;

  case 4: // Calculate 
    // Compute diameter in wavelengths
    if ((freq == 0) || (wd == -1)) {
      fl_message("Please enter the resonant frequency\n" \
		 "and the wire diameter.  Then click the\n" \
		 "Calculate button.");
    }
    else {

      dw = get_diam_wl(freq, wd, units, &wd_in);

      // Sanity check
      d1 = 0.4342945 * log(dw);

      if (d1 < -6) {  // Wire diameter less than 1E-6 wavelengths, results uncertain
	fl_message("Wire diameter is too small - results uncertain.");
      }
      if (d1 > -2) {  // Wire diameter greater than 1E-2 wavelengths, results uncertain
	fl_message("Wire diameter is too large - results uncertain.");
      }
    
      a = (aa * pow(d1,2)) + (ab * d1) + ac;
      b = (ba * pow(d1,2)) + (bb * d1) + bc;
      c = (ca * pow(d1,2)) + (cb * d1) + cc;
      d = (da * d1) + db;
      e = b + c + d;

      if (Feet_button->value()) {
	snprintf(a_str, 20, "%.3f", a*wi/12);
	snprintf(b_str, 20, "%.3f", b*wi/12);
	snprintf(c_str, 20, "%.3f", c*wi/12);
	snprintf(d_str, 20, "%.3f", d*wi/12);
	snprintf(e_str, 20, "%.3f", e*wi/12);
      }

      if (Inches_button->value()) {
	snprintf(a_str, 20, "%.3f", a*wi);
	snprintf(b_str, 20, "%.3f", b*wi);
	snprintf(c_str, 20, "%.3f", c*wi);
	snprintf(d_str, 20, "%.3f", d*wi);
	snprintf(e_str, 20, "%.3f", e*wi);
      }

      if (Meters_button->value()) {
	snprintf(a_str, 20, "%.3f", a*wmm/1000);
	snprintf(b_str, 20, "%.3f", b*wmm/1000);
	snprintf(c_str, 20, "%.3f", c*wmm/1000);
	snprintf(d_str, 20, "%.3f", d*wmm/1000);
	snprintf(e_str, 20, "%.3f", e*wmm/1000);
      }

      if (Millimeters_button->value()) {
	snprintf(a_str, 20, "%.3f", a*wmm);
	snprintf(b_str, 20, "%.3f", b*wmm);
	snprintf(c_str, 20, "%.3f", c*wmm);
	snprintf(d_str, 20, "%.3f", d*wmm);
	snprintf(e_str, 20, "%.3f", e*wmm);
      }

      // Now, let's put the results in the GUI

      Dim_A->value(a_str);
      Dim_B->value(b_str);
      Dim_C->value(c_str);
      Dim_D->value(d_str);
      Dim_E->value(e_str);

    }

    break;

  // BOZO: The spaces in these labels are there to cover the video artifacts
  //       of drawing a larger label, and then having parts of it left over
  //       when later drawing a smaller label.  This is probably a bad way
  //       to handle the problem, and I suspect won't work with different
  //       fonts and sizes.  What is a better way to handle this problem?

  case 5: // Feet radio button
    Box_A->copy_label("ft     ");
    Box_B->copy_label("ft     ");
    Box_C->copy_label("ft     ");
    Box_D->copy_label("ft     ");
    Box_E->copy_label("ft     ");
    if (!((freq == 0) || (wd == 0))) {
      ((SimpleWindow *)v)->cb_Calculate_i(w, v, 4);
    }
    break;

  case 6: // Inches radio button
    Box_A->copy_label("in     ");
    Box_B->copy_label("in     ");
    Box_C->copy_label("in     ");
    Box_D->copy_label("in     ");
    Box_E->copy_label("in     ");
    if (!((freq == 0) || (wd == 0))) {
      ((SimpleWindow *)v)->cb_Calculate_i(w, v, 4);
    }
    break;

  case 7: // meters radio button
    Box_A->copy_label("m     ");
    Box_B->copy_label("m     ");
    Box_C->copy_label("m     ");
    Box_D->copy_label("m     ");
    Box_E->copy_label("m     "); 
    if (!((freq == 0) || (wd == 0))) {
      ((SimpleWindow *)v)->cb_Calculate_i(w, v, 4);
    }
    break;

  case 8: // millimeters radio button
    Box_A->copy_label("mm ");
    Box_B->copy_label("mm ");
    Box_C->copy_label("mm ");
    Box_D->copy_label("mm ");
    Box_E->copy_label("mm ");
    if (!((freq == 0) || (wd == 0))) {
      ((SimpleWindow *)v)->cb_Calculate_i(w, v, 4);
    }
    break;

  default:
    break;

  }
}

//-----------------------------------------------
// Callback: HelpAbout
//-----------------------------------------------

void SimpleWindow::cb_HelpAbout (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_HelpAbout_i(v);
}

void SimpleWindow::cb_HelpAbout_i(void *v) {
  Fl_Help_Dialog *help_about;

  help_about = new Fl_Help_Dialog();
  help_about->value("<html><body><h1>About Fl_MoxGen</h1>"                       \
		    "<p>Fl_MoxGen is a program which will compute the "          \
		    "dimensions of a Moxon Rectangle antenna, and output "       \
		    "a file "	                                                 \
		    "suitable for the xnec2c antenna modeling software.</p>"     \
		    "<p>Version:  "					         \
		    VERSION						         \
		    "</p><p>This program is Free Software under the terms "      \
		    "of the GNU General Public License, version 3 (or "          \
		    "later).</p>"					         \
		    "<p>Algorithms and diagram by L.B. Cebik (W4RNL SK)</p>"     \
		    "<p>Standard wire gauge code contributed by "                \
		    "Colin Tuckley (G8TMV)</p>"                                  \
		    "<p>Look and feel inspired by Dan Maguire (AC6LA)</p>"       \
		    "<p>Windows version compiled by Rich Crisafulli (AB1HD)</p>" \
		    "<p>Copyright 2009 Andy Stewart (KB1OIQ)</p>"	         \
		    );
  help_about->show();

}

//-----------------------------------------------
// Callback: HelpGettingStarted
//-----------------------------------------------

void SimpleWindow::cb_HelpGettingStarted (Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_HelpGettingStarted_i(v);
}

void SimpleWindow::cb_HelpGettingStarted_i(void *v) {
  Fl_Help_Dialog *help;

  help = new Fl_Help_Dialog();
  help->value("<html><body><h1>Getting Started with Fl_MoxGen</h1>" \
	      "<p>This program is an aid to antenna builders.  " \
	      "Fl_MoxGen will compute the dimensions of one " \
	      "particular type of antenna called a Moxon Rectangle.</p>" \
	      \
	      "<h2>Enter the Required Data</h2> " \
	      "<p>The user must supply two pieces of data:" \
	      "<p><ul><li>the resonant frequency in MHz</li>" \
	      "<li>the diameter of the wire (or other construction material)</li></ul>" \
	      "</p><p>The wire size (diameter) may be expressed in the following terms:" \
	      "<p><ul><li>AWG (American Wire Gauge)</li>" \
	      "<li>SWG (Standard Wire Gauge) 0 - 20</li>" \
	      "<li>inches (in)</li>" \
	      "<li>millimeters (mm)</li></ul></p>" \
	      "<p>This type of antenna is typically built out of solid " \
	      "copper wire, rod, or tubing.</p>" \
	      \
	      "<h2>Compute the Antenna Dimensions</h2> " \
	      "<p>Once the required data are entered " \
	      "into Fl_MoxGen, press the Calculate button to compute " \
	      "the dimensions of the antenna.  The \"Result Units\" " \
	      "buttons can be clicked by the user to specify the " \
	      "desired units for the dimensions of the antenna.</p>" \
	      "<h2>Output the Results</h2> " \
	      "<p>With the data suitably displayed, the user may " \
	      "create a PDF file.  " \
	      "This PDF file will contain a diagram of the antenna, " \
	      "labels for each dimension, the resonant frequency, " \
	      "the diameter of the construction material, and " \
	      "the antenna dimensions.</p>" \
	      "<p>To generate the PDF file, click on the " \
	      "\"File->Print to File\" menu.  A dialog box will appear " \
	      "which will allow the user to choose a file name and " \
	      "a destination directory for the PDF output.  Click OK " \
	      "and the file will be written.  Click Cancel and no output " \
	      "will be generated.</p>" \
	      \
	      "<h2>Create a File for Antenna Modeling</h2>" \
	      "<p>Fl_MoxGen can output a file suitable for use with " \
	      "an antenna modeling program called xnec2c.  The user " \
	      "may choose between horizontal or vertical polarization.  " \
	      "This feature is accessed via the \"File->Write Model\" " \
	      "submenus.</p>" \
	      "<p>Once the user selects either horizontal " \
	      "or vertical polarization, a dialog box will appear.  " \
	      "The user then chooses the name of the output file and " \
	      "the destination directory.  Click OK and the file will "\
	      "be written.  Click Cancel and no output will be generated.</p>" \
	      \
	      "<h2>Exiting Fl_MoxGen</h2>" 
	      "<p>There are two ways to cleanly exit the program:" \
	      "<ul><li>Click the \"File->Quit\" menu</li>" \
	      "<li>Click the Quit button</li></ul></p>" \
	      \
	      "</body></html>");
  help->show();
}

//-----------------------------------------------
// WriteModelHoriz
//-----------------------------------------------

void SimpleWindow::cb_WriteModelHoriz(Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_WriteModelHoriz_i(v);
}

void SimpleWindow::cb_WriteModelHoriz_i(void *v) {
  double a = atof( Frequency->value() );
  double b = atof( WireSize->value() );

  if ((a == 0.0) ||
      (b == 0.0)) {
    fl_message("Please enter the resonant frequency\n" \
	       "and the wire diameter.  Then click the\n" \
	       "Calculate button.  After that, feel\n" \
	       "free to generate an antenna model.");
  }
  else {
    Fl_File_Chooser *FileChooser;
    FileChooser = new Fl_File_Chooser("model_horiz.nec", 
				      "*.nec", 
				      Fl_File_Chooser::CREATE, 
				      "Horiz Model Output File");
    FileChooser->preview(0);
    FileChooser->show();
    while (FileChooser->shown()) {
      Fl::wait();
    }

    /* If the user hits cancel, the FileChooser */
    /* returns a null value.                    */

    if (FileChooser->value()) {
      gen_xnec2c_model(FileChooser->value(), 1);   // 1 for horiz model
    }
  }
}

void SimpleWindow::gen_xnec2c_model(const char *filename, int horiz) {
  FILE *outfile;
  static char mesg[250];
  static char model_units_str[20];
  static char date_str[9];
  static char time_str[9];

  static time_t rawtime;
  static struct tm *timeinfo;

  static double step_size;      // in MHz
  static double freq_range_lo;  // in MHz
  static double freq_range_hi;  // in MHz
  static double conv_factor;    // convert units to meters
  static double wire_radius;    // wire radius in output units

  static const int num_segments = 25;     // make sure this is an odd number
  static const int num_data_points = 75;

  double a = atof( Dim_A->value() );
  double b = atof( Dim_B->value() );
  double d = atof( Dim_D->value() );
  double e = atof( Dim_E->value() );
  double freq = atof( Frequency->value() );

  double half_a = a / 2;

  freq_range_lo = freq * 0.95;
  freq_range_hi = freq * 1.05;
  step_size = (freq_range_hi - freq_range_lo) / num_data_points;

  if (Inches_button->value()) {
    conv_factor = 0.0254;
    snprintf(model_units_str, 20, "%s", "INCHES");
    wire_radius = wd_in / 2;           // diameter inches to radius
  }
  if (Feet_button->value()) {
    conv_factor = 0.3048;
    snprintf(model_units_str, 20, "%s", "FEET");
    wire_radius = (wd_in / 12) / 2;    // diameter inches to feet to radius
  }
  if (Meters_button->value()) {
    conv_factor = 1.0;
    snprintf(model_units_str, 20, "%s", "METERS");
    wire_radius = (wd_in / 39.37) / 2; // diameter inches to meters to radius
  }
  if (Millimeters_button->value()) {
    conv_factor = 0.001;
    snprintf(model_units_str, 20, "%s", "MILLIMETERS");
    wire_radius = (wd_in * 25.4) / 2;  // diameter inches to mm to radius
  }

  //-----------------------------------------------
  // OK, now output the model for xnec2c!
  //-----------------------------------------------

  time (&rawtime);
  timeinfo = localtime(&rawtime);

  outfile = fopen(filename, "w");
  if (!outfile) {
    fl_alert("Error opening file %s\n%s", filename, strerror(errno));
  }
  else {
    fprintf(outfile, "CM Moxon Rectangle model created by Fl_MoxGen version %s\n", VERSION);
    fprintf(outfile, "CM %s", asctime(timeinfo));
    fprintf(outfile, "CM This model will work with xnec2c on Linux.\n");
    fprintf(outfile, "CM The dimensions are in %s.\n", model_units_str);
    fprintf(outfile, "CM Fl_MoxGen is Free Software by Andy Stewart (KB1OIQ)\n");
    fprintf(outfile, "CM Algorithms and diagram L.B. Cebik W4RNL (SK)\n");
    fprintf(outfile, "CE End Comments\n");

    //-----------------------------------------------
    // If the user chooses horizontal polarization,
    // write that model.
    //-----------------------------------------------

    if (horiz) {
      
      // Reflector long side...
      fprintf(outfile, "GW     1 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      0.0, half_a, 0.0, 
	      0.0, 0-half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     1     1 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Reflector left side
      fprintf(outfile, "GW     2 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      d, 0-half_a, 0.0,
	      0.0, 0-half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     2     2 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Reflector right side
      fprintf(outfile, "GW     3 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      d, half_a, 0.0,
	      0.0, half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     3     3 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Driven element long side
      fprintf(outfile, "GW     4 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, 0-half_a, 0.0,
	      e, half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     4     4 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Driven element left side
      fprintf(outfile, "GW     5 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, 0-half_a, 0.0,
	      e-b, 0-half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     5     5 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Driven element right side
      fprintf(outfile, "GW     6 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, half_a, 0.0,
	      e-b, half_a, 0.0,
	      wire_radius);

      fprintf(outfile, "GS     6     6 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Misc
      fprintf(outfile, "GE     0     0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "EX     0     4 %5d      0  1.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n", (num_segments/2)+1);

      fprintf(outfile, "LD     5     0     0      0  3.70000E+07  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "NH     0     0     0      0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "NE     0    20    15      1 -1.40000E+00 -1.40000E+00  5.00000E-02  2.00000E-01  2.00000E-01  0.00000E+00\n");

      fprintf(outfile, "RP     0    37    73   1000  0.00000E+00  0.00000E+00  5.00000E+00  5.00000E+00  0.00000E+00  0.00000E+00\n");

      // Frequency card
      fprintf(outfile, "FR     0    %d     0      0 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E \n",
	      num_data_points, freq_range_lo, step_size, freq_range_hi,
	      0.0, 0.0, 0.0);  

      fprintf(outfile, "EN     0     0     0      0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fclose(outfile);

      snprintf(mesg, 250, "File %s has been written.", filename);
      fl_message(mesg, 200, 400);  // note 1024 character limit on fl_message

    } else {

    //-----------------------------------------------
    // Generate vertical polarized model here!
    //-----------------------------------------------

      // Reflector long side...
      fprintf(outfile, "GW     1 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      0.0, 0.0, half_a, 
	      0.0, 0.0, 0-half_a,
	      wire_radius);

      fprintf(outfile, "GS     1     1 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Reflector left side
      fprintf(outfile, "GW     2 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      d, 0.0, 0-half_a,
	      0.0, 0.0, 0-half_a,
	      wire_radius);

      fprintf(outfile, "GS     2     2 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Reflector right side
      fprintf(outfile, "GW     3 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      d, 0.0, half_a,
	      0.0, 0.0, half_a,
	      wire_radius);

      fprintf(outfile, "GS     3     3 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Driven element long side
      fprintf(outfile, "GW     4 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, 0.0, 0-half_a,
	      e, 0.0, half_a,
	      wire_radius);

      fprintf(outfile, "GS     4     4 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

    // BOZO finish 
      // Driven element left side
      fprintf(outfile, "GW     5 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, 0.0, 0-half_a,
	      e-b, 0.0, 0-half_a,
	      wire_radius);

      fprintf(outfile, "GS     5     5 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Driven element right side
      fprintf(outfile, "GW     6 %5d % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      num_segments,
	      e, 0.0, half_a,
	      e-b, 0.0, half_a,
	      wire_radius);

      fprintf(outfile, "GS     6     6 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E\n",
	      conv_factor, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

      // Misc
      fprintf(outfile, "GE     0     0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "EX     0     4 %5d      0  1.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n", (num_segments/2)+1);

      fprintf(outfile, "LD     5     0     0      0  3.70000E+07  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "NH     0     0     0      0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fprintf(outfile, "NE     0    20    15      1 -1.40000E+00 -1.40000E+00  5.00000E-02  2.00000E-01  2.00000E-01  0.00000E+00\n");

      fprintf(outfile, "RP     0    37    73   1000  0.00000E+00  0.00000E+00  5.00000E+00  5.00000E+00  0.00000E+00  0.00000E+00\n");

      // Frequency card
      fprintf(outfile, "FR     0    %d     0      0 % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E % 7.5E \n",
	      num_data_points, freq_range_lo, step_size, freq_range_hi,
	      0.0, 0.0, 0.0);  

      fprintf(outfile, "EN     0     0     0      0  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00  0.00000E+00\n");

      fclose(outfile);

      snprintf(mesg, 250, "File %s has been written.", filename);
      fl_message(mesg, 200, 400);  // note 1024 character limit on fl_message
    }
  }
}

//-----------------------------------------------
// WriteModelVert
//-----------------------------------------------

void SimpleWindow::cb_WriteModelVert(Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_WriteModelVert_i(v);
}

void SimpleWindow::cb_WriteModelVert_i(void *v) {
  double a = atof( Frequency->value() );
  double b = atof( WireSize->value() );

  if ((a == 0.0) ||
      (b == 0.0)) {
    fl_message("Please enter the resonant frequency\n" \
	       "and the wire diameter.  Then click the\n" \
	       "Calculate button.  After that, feel\n" \
	       "free to generate an antenna model.");
  }
  else {
    Fl_File_Chooser *FileChooser;
    FileChooser = new Fl_File_Chooser("model_vert.nec", 
				      "*.nec", 
				      Fl_File_Chooser::CREATE, 
				      "Vert Model Output File");
    FileChooser->preview(0);
    FileChooser->show();
    while (FileChooser->shown()) {
      Fl::wait();
    }

    /* If the user hits cancel, the FileChooser */
    /* returns a null value.                    */

    if (FileChooser->value()) {
      gen_xnec2c_model(FileChooser->value(), 0);  // 0 for vert model
    }
  }
}

void SimpleWindow::cb_WritePDF(Fl_Widget *w, void *v) {
  ((SimpleWindow *)v)->cb_WritePDF_i(v);
}

void SimpleWindow::cb_WritePDF_i(void *v) {

#ifdef USING_LIB_HARU
  static char mesg[255];

  double a = atof( Frequency->value() );
  double b = atof( WireSize->value() );

  if ((a == 0.0) ||
      (b == 0.0)) {
    fl_message("Please enter the resonant frequency\n" \
	       "and the wire diameter.  Then click the\n" \
	       "Calculate button.  After that, feel\n" \
	       "free to generate a PDF file.");
  }
  else {

    Fl_File_Chooser *FileChooser;
    FileChooser = new Fl_File_Chooser("fl_moxgen.pdf", 
				      "*.pdf", 
				      Fl_File_Chooser::CREATE, 
				      "PDF Output File");
    FileChooser->preview(0);
    FileChooser->show();
    while (FileChooser->shown()) {
      Fl::wait();
    }

    /* If the user hits cancel, the FileChooser */
    /* returns a null value.                    */

    if (FileChooser->value()) {

      if (!(write_pdf(FileChooser->value(),     // output file name
		      Frequency->value(),       // freq
		      WireSize->value(),        // wire dimension
		      get_wire_size_units(),    // wire size units
		      get_output_dimensions(),  // output dimensions
		      Dim_A->value(),           // Dim_A
		      Dim_B->value(),           // Dim_B
		      Dim_C->value(),           // Dim_C
		      Dim_D->value(),           // Dim_D
		      Dim_E->value()            // Dim_E
		      ))) {

	snprintf(mesg, 255, "File %s has been written.", 
		 FileChooser->value());

	fl_message(mesg, 200, 400);  // note 1024 character limit on fl_message
      }
      else {
	fl_alert("Unexpected error occurred while attempting\nto generate the PDF file.");
      }
    }
  }

#else
  fl_alert("Cannot generate a PDF output file.\nThis program was not compiled with libHaru.\nDownload it from http://www.libharu.org");
#endif

}

//-----------------------------------------------
// get_wire_size_units
//-----------------------------------------------

const char *SimpleWindow::get_wire_size_units() {
  const char *ret_val;

  int wire_size_unit = WireSizeUnits->value();

  switch (wire_size_unit) {
    case 0: {
      ret_val = "AWG";
      break;
    }
    case 1: {
      ret_val = "in";
      break;
    }
    case 2: {
      ret_val = "mm";
      break;
    }
    default: {
      ret_val = "???";
      printf("PROGRAMMING ERROR: wire_size_unit = %d\n", wire_size_unit);
    }
  }
  return ret_val;
}

//-----------------------------------------------
// get_output_dimensions
//-----------------------------------------------

const char *SimpleWindow::get_output_dimensions() {
  const char *ret_val;

  if (Feet_button->value()) {
    ret_val = "ft";
  }
  else if (Inches_button->value()) {
    ret_val = "in";
  }
  else if (Millimeters_button->value()) {
    ret_val = "mm";
  }
  else if (Meters_button->value()) {
    ret_val = "m";
  }
  else {
    ret_val = "???";
    printf("PROGRAMMING ERROR: no radio button pressed??\n");
  }
  return ret_val;
}
