/**************************************************/
/* Function: write_pdf()                          */
/**************************************************/

#include <hpdf.h>
#include <setjmp.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h> // for stat()

#include "fl_moxgen_defines.h"
#include "fl_moxgen_version.h"

#define NUM_CHARS 255

/*********************************************************/
/* Set your page size and orientation accordingly        */
/*   #define PAGE_ORIENTATION HPDF_PAGE_PORTRAIT         */
/*   #define PAGE_ORIENTATION HPDF_PAGE_LANDSCAPE        */
/*                                                       */
/* In the author's opinion, the landscape mode looks     */
/* best.  For all 10 permutations (below), the image     */
/* and letters at least make it onto the page looking    */
/* somewhat reasonable but probably not perfect.  :-)    */
/*                                                       */
/*********************************************************/

#define PAGE_SIZE HPDF_PAGE_SIZE_LETTER
/* #define PAGE_SIZE HPDF_PAGE_SIZE_LEGAL */
/* #define PAGE_SIZE HPDF_PAGE_SIZE_A3 */
/* #define PAGE_SIZE HPDF_PAGE_SIZE_A4 */
/* #define PAGE_SIZE HPDF_PAGE_SIZE_B4 */

/* #define PAGE_ORIENTATION HPDF_PAGE_PORTRAIT */
#define PAGE_ORIENTATION HPDF_PAGE_LANDSCAPE

/************************************************************************/
/*  Value                      Size (mm or inches)    Size (in pixels)  */
/*  ------------------------------------------------------------------- */
/*  HPDF_PAGE_SIZE_LETTER      8.5 x 11 (inches)         612 x 792      */
/*  HPDF_PAGE_SIZE_LEGAL       8.5 x 14 (inches)         612 x 1008     */
/*  HPDF_PAGE_SIZE_A3          297 x 420 (mm)         841.89 x 1199.551 */
/*  HPDF_PAGE_SIZE_A4          210 x 297 (mm)        595.276 x 841.89   */
/*  HPDF_PAGE_SIZE_B4          250 x 353 (mm)        708.661 x 1000.63  */
/************************************************************************/

jmp_buf env;
int error_detected;

/**************************************************/
/* Error handling function                        */
/**************************************************/

#ifdef HPDF_DLL
void __stdcall
#else
void
#endif
     error_handler  (HPDF_STATUS   error_no,
		     HPDF_STATUS   detail_no,
		     void         *user_data) {

  printf ("ERROR: error_no=%04X, detail_no=%u\n",
	  (HPDF_UINT)error_no,
	  (HPDF_UINT)detail_no);

  error_detected = 1;

  longjmp(env, 1);

}

/**************************************************/
/* write_pdf function                             */
/**************************************************/

int write_pdf(const char *filename,
	      const char *freq,
	      const char *wire_dim,
	      const char *wire_dim_str,
	      const char *output_dimension,
	      const char *dim_A,
	      const char *dim_B,
	      const char *dim_C,
	      const char *dim_D,
	      const char *dim_E) {

  HPDF_Doc pdf;
  HPDF_Page page;
  HPDF_Image image;
  HPDF_Font font;

  int v,w,x,y;

  int text_width;
  char cwd[NUM_CHARS];
  struct stat buf;

  char filename_png[NUM_CHARS];
  char hdr1[NUM_CHARS];
  char hdr2[NUM_CHARS];

  char dim_a_text[NUM_CHARS];
  char dim_b_text[NUM_CHARS];
  char dim_c_text[NUM_CHARS];
  char dim_d_text[NUM_CHARS];
  char dim_e_text[NUM_CHARS];

  char trailer_text[NUM_CHARS];

  /*  printf("Got to write_pdf() - hurray!\n"); */

  error_detected = 0;

/**************************************************/
/* Setup the PDF page                             */
/**************************************************/

  pdf = HPDF_New(error_handler, NULL); 
  if (!pdf) {
    printf ("ERROR: cannot create pdf object.\n");
    return 1;
  }
 
  if (setjmp(env)) {
    HPDF_Free(pdf);
    return 1;
  }

/**************************************************/
/* Add page header - 1st line                     */
/**************************************************/

  page = HPDF_AddPage(pdf); 
  HPDF_Page_SetSize(page, PAGE_SIZE, PAGE_ORIENTATION);

  /* create default-font */
  font = HPDF_GetFont(pdf, "Helvetica", NULL);

  /*  strncpy(hdr1, "Moxon Rectangle Antenna", NUM_CHARS); */
  snprintf(hdr1, NUM_CHARS, 
	   "Moxon Rectangle Antenna for %s MHz with %s %s wire",
	   freq, wire_dim, wire_dim_str);

  HPDF_Page_BeginText(page);
  HPDF_Page_SetFontAndSize(page, font, 18);

  text_width = (int) HPDF_Page_TextWidth(page, hdr1);
  HPDF_Page_MoveTextPos(page, 
			(HPDF_Page_GetWidth(page) - text_width) / 2,
			(int)HPDF_Page_GetHeight(page) - 50);

  HPDF_Page_ShowText(page, hdr1);
  HPDF_Page_EndText(page);

/**************************************************/
/* Put the moxgen.png image into the PDF document */
/* The file may live in the current working       */
/* directory, if it has not yet been installed,   */
/* or it will live in SHAREDIR.  Check both       */
/* locations.                                     */
/**************************************************/

  getcwd(cwd, sizeof(cwd));
  snprintf(filename_png, NUM_CHARS, "%s/%s", cwd, "moxgen.png");

  if (!stat(filename_png, &buf)) {
    image = HPDF_LoadPngImageFromFile(pdf, filename_png);
  }
   else {
     snprintf(filename_png, NUM_CHARS, "%s/%s", SHAREDIR, "moxgen.png");
     if (!stat(filename_png, &buf)) {
       image = HPDF_LoadPngImageFromFile(pdf, filename_png);
     }
     else {
       fprintf(stderr, "ERROR: cannot find moxgen.png - is the program properly installed?\n");
     }
   }

  x =((int)HPDF_Page_GetWidth(page) - HPDF_Image_GetWidth(image)) / 2;
  y =(int)HPDF_Page_GetHeight(page) - HPDF_Image_GetHeight(image) - 75;

  /* Draw image to the canvas. */
  HPDF_Page_DrawImage(page, image, x, y, 
		       HPDF_Image_GetWidth(image),
		       HPDF_Image_GetHeight(image));

/**************************************************/
/* Add antenna dimensions                         */
/**************************************************/

  HPDF_Page_BeginText(page);
  HPDF_Page_SetFontAndSize(page, font, 14);

  HPDF_Page_MoveTextPos(page, x+20, y-25);
  snprintf(dim_a_text, NUM_CHARS, "A = %s %s", dim_A, output_dimension);
  HPDF_Page_ShowText(page, dim_a_text);

  HPDF_Page_MoveTextPos(page, 0, -20);
  snprintf(dim_b_text, NUM_CHARS, "B = %s %s", dim_B, output_dimension);
  HPDF_Page_ShowText(page, dim_b_text);

  HPDF_Page_MoveTextPos(page, 0, -20);
  snprintf(dim_c_text, NUM_CHARS, "C = %s %s", dim_C, output_dimension);
  HPDF_Page_ShowText(page, dim_c_text);

  HPDF_Page_MoveTextPos(page, 0, -20);
  snprintf(dim_d_text, NUM_CHARS, "D = %s %s", dim_D, output_dimension);
  HPDF_Page_ShowText(page, dim_d_text);

  HPDF_Page_MoveTextPos(page, 0, -20);
  snprintf(dim_e_text, NUM_CHARS, "E = %s %s", dim_E, output_dimension);
  HPDF_Page_ShowText(page, dim_e_text);

  HPDF_Page_EndText(page);


/**************************************************/
/* Add the page trailer                           */
/**************************************************/

  snprintf(trailer_text, NUM_CHARS, "Generated by Fl_MoxGen version %s %s", VERSION, " - Written by Andy Stewart (KB1OIQ)");

  HPDF_Page_BeginText(page);
  HPDF_Page_SetFontAndSize(page, font, 10);

  text_width = (int) HPDF_Page_TextWidth(page, trailer_text);
  HPDF_Page_MoveTextPos(page, 
			(HPDF_Page_GetWidth(page) - text_width) / 2,
			50);

  HPDF_Page_ShowText(page, trailer_text);
  HPDF_Page_EndText(page);

/**************************************************/
/* Save file and free the object                  */
/**************************************************/

  HPDF_SaveToFile(pdf, filename);
  HPDF_Free(pdf);

  return error_detected;

}
